/*   _______              __
    / ___/ /  ___  __ _  / /  ___
   / /__/ _ \/ _ \/  ' \/ _ \/ _ \
   \___/_//_/\___/_/_/_/_.__/\___/
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
//
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//

#include <iostream>
using std::cout;
using std::cin;
using std::cerr;
using std::endl;

#include "Box.H"
#include "Vector.H"
#include "DisjointBoxLayout.H"
#include "LayoutIterator.H"
#include "parstream.H"

#include "AMRLevel.H"

int AMRLevel::s_verbosity = 0;

bool AMRLevel::isDefined() const
{
  return m_isDefined;
}

AMRLevel::~AMRLevel()
{
}

AMRLevel::AMRLevel()
{
  m_coarser_level_ptr = NULL;
  m_finer_level_ptr = NULL;
  m_isDefined = false;
  m_level = 0;
  m_time = 0;
  m_dt = 0;
  m_initial_dt_multiplier = 0.1;
}

void AMRLevel::define(AMRLevel*  a_coarser_level_ptr,
                      const Box& a_problem_domain,
                      int        a_level,
                      int        a_ref_ratio)
{
  ProblemDomain physDomain(a_problem_domain);

  define(a_coarser_level_ptr, a_problem_domain, a_level, a_ref_ratio);
}

void AMRLevel::define(AMRLevel*            a_coarser_level_ptr,
                      const ProblemDomain& a_problem_domain,
                      int                  a_level,
                      int                  a_ref_ratio)
{
  if (s_verbosity >= 3)
  {
    pout() << "AMRLevel::define" << endl;
  }

  m_coarser_level_ptr = a_coarser_level_ptr;
  m_problem_domain = a_problem_domain;
  m_level = a_level;
  m_ref_ratio = a_ref_ratio;
  m_finer_level_ptr = NULL;
  m_isDefined = true;
}

void AMRLevel::finerLevelPtr(AMRLevel* a_finer_level_ptr)
{
  m_finer_level_ptr = a_finer_level_ptr;
}

void AMRLevel::dt(Real a_dt)
{
  m_dt = a_dt;
}

Real AMRLevel::dt() const
{
  return(m_dt);
}

const ProblemDomain& AMRLevel::problemDomain() const
{
  return(m_problem_domain);
}

Vector<Box> AMRLevel::boxes() const
{
  return(m_level_grids);
}

int AMRLevel::refRatio() const
{
  return(m_ref_ratio);
}

void AMRLevel::time(Real a_time)
{
  m_time = a_time;
}

Real AMRLevel::time() const
{
  return m_time;
}

void AMRLevel::initialDtMultiplier(Real a_initial_dt_multiplier)
{
  m_initial_dt_multiplier = a_initial_dt_multiplier;
}

Real AMRLevel::initialDtMultiplier() const
{
  return m_initial_dt_multiplier;
}

// static
void AMRLevel::verbosity(int a_verbosity)
{
  s_verbosity = a_verbosity;
}

// static
int AMRLevel::verbosity()
{
  return(s_verbosity);
}

void AMRLevel::postRegrid(int a_base_level)
{
  if (s_verbosity >= 3)
  {
    pout() << "AMRLevel::postRegrid" << endl;
  }
}
