/* _______              __
  / ___/ /  ___  __ _  / /  ___
 / /__/ _ \/ _ \/  ' \/ _ \/ _ \
 \___/_//_/\___/_/_/_/_.__/\___/
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
//
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//
// file: ArrayViewData.H

#ifndef _ARRAYVIEW_DATA_H_
#define _ARRAYVIEW_DATA_H_

#include "REAL.H"
#include "BaseFab.H"
#include "FArrayBox.H"
#include "LayoutData.H"

/// Special LayoutData type for use with ArrayView
/** ArrayViewData is a wrapper around LayoutData for BaseFab<Real> or
 *  FArrayBox for use in ArrayView.  It provides random access to the
 *  individual FABs and the ability to directly set the data in them.  It is
 *  intended to work only in ArrayView.  Any other attept to use this class
 *  is unsupported.
 */

class ArrayViewData
{
public:
  ArrayViewData() : m_layoutdata_ptr(NULL) {};
  ~ArrayViewData() {};
  ArrayViewData(LayoutData<BaseFab<Real> >* a_layoutdata_ptr)
    :
    m_layoutdata_ptr(a_layoutdata_ptr)
    {};

  ArrayViewData(LayoutData<FArrayBox>* a_layoutdata_ptr)
    :
    m_layoutdata_ptr((LayoutData<BaseFab<Real> >*) a_layoutdata_ptr)
    {};

  /// Access a single FAB in the LayoutData using its internal index
  BaseFab<Real>&
  operator[](int a_elem)
    {
      return *(m_layoutdata_ptr->m_vector[a_elem]);
    }

  /// Substitute a different FAB into the LayoutData
  /** The caller is responsible for managing the memory used by the FAB that
   * is being replaced.
   */
  void
  set(int a_elem, BaseFab<Real>* a_fab_ptr)
    {
      m_layoutdata_ptr->m_vector[a_elem] = a_fab_ptr;
    }

private:
  LayoutData<BaseFab<Real> >* m_layoutdata_ptr;
};

#endif
