
#include "BaseFab.H"
#include <cstring>

template < >
void
BaseFab<Real>::define()
{
  assert(nvar > 0);
  assert(dptr == 0);
  assert(numpts > 0);
  assert(!aliased);
  //    assert(!(The_FAB_Arena == 0));// not a sufficient test !!!
#ifdef ENABLE_MEMORY_TRACKING  
  if(s_Arena == NULL) s_Arena = new BArena(name().c_str());
#else
  if(s_Arena == NULL) s_Arena = new BArena("");
#endif
//   if(s_Arena == NULL)
//     MayDay::Error("malloc in basefab failed");
  truesize = nvar*numpts;
  dptr     = static_cast<Real*>(s_Arena->alloc(truesize*sizeof(Real)));

#ifdef ENABLE_MEMORY_TRACKING  
  s_Arena->bytes+=truesize*sizeof(Real)+sizeof(BaseFab<Real>);
  if(s_Arena->bytes > s_Arena->peak) 
    s_Arena->peak = s_Arena->bytes;
#endif

  // this is to be consistent with FArrayBox behavior
#ifndef NDEBUG
  setVal(1.23456789e10);
#endif
  
}


template < >
void
BaseFab<int>::define()
{
  assert(nvar > 0);
  assert(dptr == 0);
  assert(numpts > 0);
  assert(!aliased);
  //    assert(!(The_FAB_Arena == 0));// not a sufficient test !!!
#ifdef ENABLE_MEMORY_TRACKING  
  if(s_Arena == NULL) s_Arena = new BArena(name().c_str());
#else
  if(s_Arena == NULL) s_Arena = new BArena("");
#endif
//   if(s_Arena == NULL)
//     MayDay::Error("malloc in basefab failed");
  truesize = nvar*numpts;
  dptr     = static_cast<int*>(s_Arena->alloc(truesize*sizeof(int)));

#ifdef ENABLE_MEMORY_TRACKING  
  s_Arena->bytes+=truesize*sizeof(int)+sizeof(BaseFab<int>);
  if(s_Arena->bytes > s_Arena->peak) 
    s_Arena->peak = s_Arena->bytes;
#endif
}


template < >
void
BaseFab<Real>::undefine ()
{

  if(aliased)
	{
	  dptr = 0;
	  return;
	}
  if(dptr == 0) return;

  s_Arena->free(dptr);

#ifdef ENABLE_MEMORY_TRACKING  
  s_Arena->bytes-= truesize*sizeof(Real)+sizeof(BaseFab<Real>);
#endif

  dptr = 0;
}


template < >
void
BaseFab<int>::undefine ()
{

  if(aliased)
	{
	  dptr = 0;
	  return;
	}
  if(dptr == 0) return;

  s_Arena->free(dptr);

#ifdef ENABLE_MEMORY_TRACKING  
  s_Arena->bytes-= truesize*sizeof(int)+sizeof(BaseFab<int>);
#endif

  dptr = 0;
}

template < >
void
BaseFab<Real>::setVal (Real val)
{
  if(val==0)
	{
	  memset(dptr, 0, truesize*sizeof(Real));
	}
  else
	{
	  Real* end = dptr + truesize;
	  for(Real* v = dptr; v<end; v++) 
		{
		  *v=val;
		}
	}
}
