/* _______              __
  / ___/ /  ___  __ _  / /  ___
 / /__/ _ \/ _ \/  ' \/ _ \/ _ \
 \___/_//_/\___/_/_/_/_.__/\___/ 
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
// 
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//
// DTGraves

#ifndef _BOX_ITERATOR_H_
#define _BOX_ITERATOR_H_

#include <cstdlib>

#include "Box.H"
#include "REAL.H"
#include "SPACE.H"
#include "Tuple.H"
#include "IntVect.H"

///iterates through the IntVects of a Box
/**
   BoxIterator iterates through the IntVects of a box.  The actual
   sqeuence of IntVects is implementation-specific.  
   Typical usage:

   Box b; 
   ...
   BoxIterator bit (b); 
   for (bit.begin(); bit.ok(); ++bit)  
    {    
      IntVect iv = bit(); 
     (do operations involving iv)  
    }
*/
class BoxIterator
{
public:


  ///
  /**
     Default constructor.  This constructs an invalid iterator.
     The user must call define before using.
  */    
  BoxIterator ();

  ///
  /**
     Constructs a BoxIterator and associates it with a Box.
      Arguments:
     bx (not modified) the Box to iterate over.
  */
  BoxIterator(const Box& bx);

  void setBox(const Box& bx);

  ///
  /**
     Associates a Box with this BoxIterator.
     Arguments:
     bx (not modified) the Box to iterate over. 
  */
  void define (const Box& bx);

  ///
  /**
     Copy constructor.
      Arguments:
     tx (not modified) the BoxIterator to copy.
  */
  BoxIterator (const BoxIterator& tx);
    
  ///
  ~BoxIterator ();

  ///
  /**
     Sets this BoxIterator to the first IntVect in its Box.  The
     definition of the "first" IntVect is
     implementation-dependent.
  */
  void begin();

  ///
  /**
     Sets this BoxIterator to the first IntVect in its Box.  The
     definition of the "first" IntVect is
     implementation-dependent.
  */
  void reset();

  ///
  /**
     Modifies this BoxIterator to set it to the next location in its
     Box.  The definition of the "next location" of a Box is
     implementation-dependent. 
  */
  void operator++();

  void next();


  ///
  /**
     Returns the value of the InVect for the current location of this BoxIterator.
  */
  const IntVect& operator() () const;

  ///
  /**
     Returns true if this BoxIterator's location is within its Box.
  */
  bool ok();

protected:

  IntVect m_current;
  IntVect m_boxLo;
  IntVect m_boxHi;
};

#endif
