/* _______              __
  / ___/ /  ___  __ _  / /  ___
 / /__/ _ \/ _ \/  ' \/ _ \/ _ \
 \___/_//_/\___/_/_/_/_.__/\___/ 
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
// 
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//
// DTGraves

#include "BoxIterator.H"

/*******/
/*******/
BoxIterator::
BoxIterator ()
{
  m_current = IntVect::Unit;
  m_boxLo   = IntVect::Unit;
  m_boxHi   = IntVect::Zero;
}

/*******/
/*******/
BoxIterator::
BoxIterator(const Box& bx)
{
  define(bx);
}

/*******/
/*******/
void 
BoxIterator::
setBox(const Box& bx)
{
  define(bx);
}


/*******/
/*******/
void 
BoxIterator::
define (const Box& a_bx)
{
  //this algorithm breaks for 
  //screwy empty boxes
  if(a_bx.smallEnd() <= a_bx.bigEnd())
    {
      m_current = a_bx.smallEnd();
      m_boxLo   = a_bx.smallEnd();
      m_boxHi   = a_bx.bigEnd();
    }
  else
    {
      m_current = IntVect::Unit;
      m_boxLo   = IntVect::Unit;
      m_boxHi   = IntVect::Zero;
    }
}

/*******/
/*******/
BoxIterator::
BoxIterator (const BoxIterator& a_iterIn)
{
  m_current = a_iterIn.m_current;
  m_boxLo =   a_iterIn.m_boxLo;
  m_boxHi =   a_iterIn.m_boxHi;
}

/*******/
/*******/
BoxIterator::
~BoxIterator ()
{
}

/*******/
/*******/
void 
BoxIterator::
begin()
{
  if(m_boxLo <= m_boxHi)
    {
      m_current = m_boxLo;
    }
}
/*******/
/*******/

void 
BoxIterator::
reset()
{
  begin();
}
/*******/
/*******/

void 
BoxIterator::
operator++()
{
  next();
}
/*******/
/*******/
void 
BoxIterator::
next()
{
  m_current[0]++;
#if CH_SPACEDIM >= 2
  if(m_current[0] > m_boxHi[0])
    {
      m_current[0] = m_boxLo[0];
      m_current[1]++;
#if CH_SPACEDIM == 3 
      if(m_current[1] > m_boxHi[1])
        {
          m_current[1] = m_boxLo[1];
          m_current[2]++;
        }
#endif
    }
#endif
}
/*******/
/*******/
const IntVect& 
BoxIterator::
operator() () const
{
  assert(m_current <= m_boxHi);
  assert(m_current >= m_boxLo);
  return m_current;
}
/*******/
/*******/
bool 
BoxIterator::
ok()
{
  return (m_current  <= m_boxHi);
}
/*******/
/*******/
