/* _______              __
  / ___/ /  ___  __ _  / /  ___
 / /__/ _ \/ _ \/  ' \/ _ \/ _ \
 \___/_//_/\___/_/_/_/_.__/\___/ 
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
// 
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//

#include "LoHiSide.H"
#include <cassert>

Side::LoHiSide
flip(Side::LoHiSide side) 
{
    assert ( (side == Side::Lo) || (side == Side::Hi) );
    return ( (side == Side::Lo)
             ? Side::Hi
             : Side::Lo );
}

Side::LoHiSide
Side::flip(Side::LoHiSide side) 
{
    assert ( (side == Side::Lo) || (side == Side::Hi) );
    return ( (side == Side::Lo)
             ? Side::Hi
             : Side::Lo );
}


int 
sign(Side::LoHiSide side)
{
    assert ( (side == Side::Lo) || (side == Side::Hi) );
    return ( (side == Side::Lo) ? -1 : 1 );
}



SideIterator::SideIterator() : current(-1)
{    
  reset();
}


void
SideIterator::begin()
{
    current = 0;
}


bool 
SideIterator::ok() const
{
    return ( (current > -1) && (current < Side::NUMSIDES) );
}


void
SideIterator::operator++()
{
    ++current;
}

void
SideIterator::next()
{
    ++current;
}


Side::LoHiSide
SideIterator::operator() () const
{
    switch (current)
    {
    case 0:
        return Side::Lo;
    case 1:
        return Side::Hi;
    default:
        return Side::Invalid;
    }
}
