/* _______              __
  / ___/ /  ___  __ _  / /  ___
 / /__/ _ \/ _ \/  ' \/ _ \/ _ \
 \___/_//_/\___/_/_/_/_.__/\___/ 
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
// 
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//

#ifndef MEMTRACK_H
#define MEMTRACK_H

#ifdef ENABLE_MEMORY_TRACKING

#include <iostream>
using std::ostream;


///
void ReportUnfreedMemory(ostream& os);
///
void ReportMemoryUsage(ostream& os);

/// calls ReportUnfreedMemory with pout() 
void UnfreedMemory();

/// calls  ReportMemoryUsage with pout()
void MemoryUsage();  

///
void memTrackingOn();

///
void memtrackingOff();

void overallMemoryUsage(long long& currentTotal, long long& peak);

/// Memory tracking functions
/**
 Memory tracking functions
*/

class Memtrack
{
public:

  /// produce a formated output onto os of memory usage.
  /**
	 memory is reported one a class-by-class basis.
	 When ENABLE_MEMORY_TRACKING is turned on you also
	 get a report from the atexit() function.  This can
	 be handy for spotting memory leaks.  The memory tracking
	 functionality consumes very little runtime and does not
	 impede optimized performance.
  */
  static void ReportUnfreedMemory(ostream& os)
  {::ReportUnfreedMemory(os);}

/// calls ReportUnfreedMemory with pout()
  static void UnfreedMemory()
  {::UnfreedMemory();}

  static void memTrackingOn()
  {:: memTrackingOn();}

  static void memtrackingOff()
  {::memtrackingOff();}

  static void overallMemoryUsage(long long& currentTotal, long long& peak)
  {::overallMemoryUsage(currentTotal, peak);}
};

// =========== end user functions.===================================


void AddTrack(void* addr,  size_t asize,  const char *fname, 
              int lnum, bool malloc);
void RemoveTrack(void* addr, bool malloc);


// void * operator new(size_t size, char const *file, int line);

// void operator delete(void *p) throw();

// void * operator new[](size_t size, char const *file, int line);

// void operator delete[](void *p) throw();


// these are ordered in this way so that we don't double track malloc
// functions. 
void* mallocp(size_t size, const char* file, int line);
void* reallocp(void* p, size_t size, const char* file, int line);
void* callocp(size_t nelem, size_t elsize, const char* file, int line);
void  freep(void* p);

#define calloc(a,b)   callocp(a,b,__FILE__, __LINE__)
#define realloc(a, b) reallocp(a, b, __FILE__, __LINE__)
#define malloc(a)     mallocp(a, __FILE__, __LINE__)
#define free(a)       freep(a)

#endif

#endif

