/*   _______              __
    / ___/ /  ___  __ _  / /  ___
   / /__/ _ \/ _ \/  ' \/ _ \/ _ \
   \___/_//_/\___/_/_/_/_.__/\___/ 
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
// 
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//

#include "BinItem.H"

BinItem::BinItem()
  : m_position(D_DECL(1.0e8, 1.0e8, 1.0e8))
{ 
}

BinItem::BinItem(const RealVect& a_position)
  : m_position(a_position)
{
}

BinItem::~BinItem()
{
}

void BinItem::define(const RealVect& a_position)
{
  m_position = a_position;
}

void BinItem::setPosition(const RealVect& a_position)
{
  m_position = a_position;
}

void BinItem::setPosition(const Real a_position, const int a_dimension)
{
  m_position[a_dimension] = a_position;
}

const RealVect& BinItem::position() const
{
  return m_position;
}

RealVect& BinItem::position() 
{
  return m_position;
}

Real BinItem::position(const int a_dir) const
{
  return m_position[a_dir];
}

int BinItem::size() const
{
  return SpaceDim*sizeof(Real);
}

void BinItem::linearOut(void* buf) const
{
  Real* buffer = (Real*) buf;
  for (int dim=0; dim<SpaceDim; dim++) 
    {
      *buffer = m_position[dim];
      ++buffer;
    }
}

void BinItem::linearIn(void* buf) 
{
  Real* buffer = (Real*) buf;
  for (int dir=0; dir<SpaceDim; ++dir)
    {
      m_position[dir] = *buffer;
      ++buffer;
    }
}

// Write a human-readable version to stdout.  This is for debugging.
void BinItem::print(void) const
{
  std::cout << *this;
}

std::ostream& operator<<( std::ostream& ostr, const BinItem& p)
{
  ostr << "(BinItem (position " << p.position(0);
  for (int i=1; i<SpaceDim; ++i)
    {
      ostr << " " << p.position(i);
    }
  ostr << "))";

  return ostr;
}
