/* _______              __
  / ___/ /  ___  __ _  / /  ___
 / /__/ _ \/ _ \/  ' \/ _ \/ _ \
 \___/_//_/\___/_/_/_/_.__/\___/ 
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
// 
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//

#include <cmath>
#include "IntVect.H"
#include "Box.H"
#include "IntVectSet.H"
#include "DebugOut.H"

/// Global variables for test output:

static const char *pgmname = "newIVSTest" ;
static const char *indent = "   " ,*indent2 = "      " ; 
static bool verbose = true ;


/// Prototypes:

void
parseTestOptions( int argc ,char* argv[] ) ;

/**
   testIVS returns:
    0: all tests passed
 */
int testIVS(void);

int main(int argc, char* argv[])
{
#ifdef MPI
  MPI_Init(&argc, &argv);
#endif
  bool passed = true;
  int icode = 0;

  parseTestOptions( argc ,argv ) ;
  if( verbose )
    cout << indent2 << "Beginning " << pgmname << " ..." << endl ;

  icode = testIVS();
  if(icode != 0)
    {
      cout << indent << pgmname 
           << ": failed with error code " << icode
           << endl;
      passed = false;
    }

  if(passed)
    cout << indent << pgmname
           << ": passed all tests"
           << endl;
#ifdef MPI
  MPI_Finalize();
#endif

  return 0;
}

int testIVS()
{
  Box box(IntVect::Zero, IntVect::Unit);
  IntVectSet  ivs(box);
  int ncomp = 3;
  IVSFAB<Real> fab(ivs, ncomp);
  fab.setVal(7.5);
  dumpIVSFAB(&fab);
  
  Real val = 0.0;
  for(IVSIterator ivsit(ivs); ivsit.ok(); ++ivsit)
    {
      for(int icomp = 0; icomp < ncomp; icomp++)
        {
          fab(ivsit(), icomp) = val;
          val += 1.0;
        }
    }

  dumpIVSFAB(&fab);
  return 0;
} 
/*****************/
///
// Parse the standard test options (-v -q) out of the command line.
// Stop parsing when a non-option argument is found.
///
/*****************/
void
parseTestOptions( int argc ,char* argv[] ) 
{
  for( int i = 1 ; i < argc ; ++i )
    {
      if( argv[i][0] == '-' ) //if it is an option
        {
          // compare 3 chars to differentiate -x from -xx
          if( strncmp( argv[i] ,"-v" ,3 ) == 0 )
            {
              verbose = true ;
              argv[i] = "" ;
            }
          else if( strncmp( argv[i] ,"-q" ,3 ) == 0 )
            {
              verbose = false ;
              argv[i] = "" ;
            }
          else
            {
              break ;
            }
        }
    }
  return ;
}
